@@ -44,10 +44,6 @@ int gnutls_srp_set_server_credentials_file( gnutls_srp_server_credentials res,
 
 const char* gnutls_srp_server_get_username( gnutls_session state);
 
-typedef int gnutls_srp_server_select_function(gnutls_session, const char **, const char**, unsigned int);
-void gnutls_srp_server_set_select_function( gnutls_session, gnutls_srp_server_select_function *);
-
-
 int gnutls_srp_verifier( const char* username, const char* password, const gnutls_datum *salt, 
 	const gnutls_datum* g, const gnutls_datum* n, 
 	gnutls_datum * res);
