@@ -124,6 +124,69 @@ void gnutls_certificate_free_ca_names(gnutls_certificate_credentials sc)
 	_gnutls_free_datum( &sc->x509_rdn_sequence);
 }
 
+/*-
+  * _gnutls_certificate_get_dh_params - Returns the DH parameters pointer
+  * @sc: is an &gnutls_certificate_credentials structure.
+  *
+  * This function will return the dh parameters pointer. This will read the
+  * credentials structure, and cache the output to the session, so later
+  * calls would not examine the credentials (or call a callback).
+  *
+  -*/
+gnutls_dh_params _gnutls_certificate_get_dh_params(const gnutls_certificate_credentials sc,
+	gnutls_session session)
+{
+gnutls_params_st params;
+int ret;
+
+	if (session->internals.params.cert_dh_params) {
+		return session->internals.params.cert_dh_params;
+	}
+
+	if (sc->dh_params) {
+		session->internals.params.cert_dh_params = sc->dh_params;
+	} else if (sc->params_func) {
+		ret = sc->params_func( session, GNUTLS_PARAMS_DH, &params);
+		if (ret == 0 && params.type == GNUTLS_PARAMS_DH) {
+			session->internals.params.cert_dh_params = params.params.dh;
+			session->internals.params.free_cert_dh_params = params.deinit;
+		}
+	}
+	
+	return session->internals.params.cert_dh_params;
+}
+
+/*-
+  * _gnutls_certificate_get_rsa_params - Returns the RSA parameters pointer
+  * @sc: is an &gnutls_certificate_credentials structure.
+  *
+  * This function will return the rsa parameters pointer.
+  *
+  -*/
+gnutls_rsa_params _gnutls_certificate_get_rsa_params(const gnutls_certificate_credentials sc,
+	gnutls_session session)
+{
+gnutls_params_st params;
+int ret;
+
+	if (session->internals.params.rsa_params) {
+		return session->internals.params.rsa_params;
+	}
+
+	if (sc->rsa_params) {
+		session->internals.params.rsa_params = sc->rsa_params;
+	} else if (sc->params_func) {
+		ret = sc->params_func( session, GNUTLS_PARAMS_RSA_EXPORT, &params);
+		if (ret == 0 && params.type == GNUTLS_PARAMS_RSA_EXPORT) {
+			session->internals.params.rsa_params = params.params.rsa_export;
+			session->internals.params.free_rsa_params = params.deinit;
+		}
+	}
+	
+	return session->internals.params.rsa_params;
+}
+
+
 /**
   * gnutls_certificate_free_credentials - Used to free an allocated gnutls_certificate_credentials structure
   * @sc: is an &gnutls_certificate_credentials structure.
@@ -240,84 +303,6 @@ void gnutls_certificate_server_set_request(gnutls_session session,
 	session->internals.send_cert_req = req;
 }
 
-
-#define gnutls_certificate_client_select_function certificate_client_select_func
-
-/**
-  * gnutls_certificate_client_set_select_function - Used to set a callback while selecting the proper (client) certificate
-  * @session: is a &gnutls_session structure.
-  * @func: is the callback function
-  *
-  * This function sets a callback to be called while selecting the (client) certificate.
-  * The callback's function prototype is:
-  * int (*callback)(gnutls_session, const gnutls_datum *client_cert, int ncerts, const gnutls_datum* req_ca_dn, int nreqs);
-  *
-  * @client_cert contains @ncerts gnutls_datum structures which hold
-  * the raw certificates (DER for X.509 or binary for OpenPGP), of the
-  * client.
-  *
-  * @req_ca_dn, is only used in X.509 certificates. 
-  * Contains a list with the CA names that the server considers trusted. 
-  * Normally we should send a certificate that is signed
-  * by one of these CAs. These names are DER encoded. To get a more
-  * meaningful value use the function gnutls_x509_rdn_get().
-  *
-  * This function specifies what we, in case of a client, are going
-  * to do when we have to send a certificate. If this callback
-  * function is not provided then gnutls will automaticaly try to
-  * find an appropriate certificate to send. The appropriate certificate
-  * is chosen based on the CAs sent by the server, and the requested
-  * public key algorithms.
-  *
-  * If the callback function is provided then gnutls will call it, in the
-  * handshake, after the certificate request message has been received.
-  *
-  * The callback function should return the index of the certificate
-  * choosen by the user. The index is relative to the certificates in the
-  * callback's parameter. The value (-1) indicates that the user
-  * does not want to use client authentication.
-  **/
-void gnutls_certificate_client_set_select_function(gnutls_session session,
-					     gnutls_certificate_client_select_function
-					     * func)
-{
-	session->internals.client_cert_callback = func;
-}
-
-#define gnutls_certificate_server_select_function certificate_server_select_func
-/**
-  * gnutls_certificate_server_set_select_function - Used to set a callback while selecting the proper (server) certificate
-  * @session: is a &gnutls_session structure.
-  * @func: is the callback function
-  *
-  * This function sets a callback to be called while selecting the (server) certificate.
-  * The callback's function form is:
-  * int (*callback)(gnutls_session, gnutls_datum *server_cert, int ncerts);
-  *
-  * @server_cert contains @ncerts gnutls_datum structures which hold
-  * the raw certificate (DER encoded in X.509) of the server. 
-  *
-  * This function specifies what we, in case of a server, are going
-  * to do when we have to send a certificate. If this callback
-  * function is not provided then gnutls will automaticaly try to
-  * find an appropriate certificate to send. (actually send the first in the list)
-  *
-  * In case the callback returned a negative number then gnutls will
-  * not attempt to choose the appropriate certificate and the caller function
-  * will fail.
-  *
-  * The callback function will only be called once per handshake.
-  * The callback function should return the index of the certificate
-  * choosen by the server. -1 indicates an error.
-  *
-  **/
-void gnutls_certificate_server_set_select_function(gnutls_session session,
-					     gnutls_certificate_server_select_function
-					     * func)
-{
-	session->internals.server_cert_callback = func;
-}
-
 /**
   * gnutls_certificate_client_set_retrieve_function - Used to set a callback to retrieve the certificate
   * @cred: is a &gnutls_certificate_credentials structure.
