@@ -24,22 +24,19 @@
 #include <gnutls_compress.h>
 #include <gnutls_algorithms.h>
 #include "gnutls_errors.h"
-
-#ifdef USE_LZO
-# ifdef USE_MINILZO
-#  include "../libextra/minilzo.h" /* get the prototypes only.
+#ifdef USE_MINILZO
+# include "../libextra/minilzo.h" /* get the prototypes only.
       *	Since LZO is a GPLed library, the gnutls_global_init_extra() has
       *	to be called, before LZO compression can be used.
       */
-# else
-#  include <lzo1x.h>
-# endif
+#else
+# include <lzo1x.h>
+#endif
 
 typedef int (*LZO_FUNC)();
 
 LZO_FUNC _gnutls_lzo1x_decompress_safe = NULL;
 LZO_FUNC _gnutls_lzo1x_1_compress = NULL;
-#endif
 
 /* The flag d is the direction (compress, decompress). Non zero is
  * decompress.
@@ -60,7 +57,7 @@ int err;
 
 	switch( method) {
 #ifdef HAVE_LIBZ
-	    case GNUTLS_COMP_DEFLATE: {
+	    case GNUTLS_COMP_ZLIB: {
 		int window_bits, mem_level;
 		int comp_level;
 		z_stream* zhandle;
@@ -96,7 +93,6 @@ int err;
 		break;
 	    }
 #endif
-#ifdef USE_LZO
 	    case GNUTLS_COMP_LZO:
 	        if (d) /* LZO does not use memory on decompressor */
 	        { /* ret->handle = NULL; */ }
@@ -112,7 +108,6 @@ int err;
 		}
 		
 		break;
-#endif
 	}
 	return ret;
 }
@@ -125,7 +120,7 @@ int err;
 			/* case GNUTLS_COMP_LZO:
 				break; */
 #ifdef HAVE_LIBZ
-			case GNUTLS_COMP_DEFLATE:
+			case GNUTLS_COMP_ZLIB:
 				if (d)
 					err = inflateEnd( handle->handle);
 				else
@@ -152,7 +147,6 @@ int err;
 	 */
 	
 	switch( handle->algo) {
-#ifdef USE_LZO
 		case GNUTLS_COMP_LZO: {
 			lzo_uint out_len;
 			size_t size;
@@ -179,9 +173,8 @@ int err;
 			compressed_size = out_len;
 			break;
 		}		
-#endif
 #ifdef HAVE_LIBZ
-		case GNUTLS_COMP_DEFLATE: {
+		case GNUTLS_COMP_ZLIB: {
 			uLongf size;
 			z_stream *zhandle;
 			
@@ -245,7 +238,6 @@ int cur_pos;
 	 */
 	
 	switch(handle->algo) {
-#ifdef USE_LZO
 		case GNUTLS_COMP_LZO: {
 			lzo_uint out_size;
 			lzo_uint new_size;
@@ -280,9 +272,9 @@ int cur_pos;
 			plain_size = new_size;
 			break;
 		}
-#endif
+
 #ifdef HAVE_LIBZ
-		case GNUTLS_COMP_DEFLATE: {
+		case GNUTLS_COMP_ZLIB: {
 			uLongf out_size;
 			z_stream* zhandle;
 
